from loguru import logger
import mercantile
import owntwin.builder.utils as utils
from owntwin.builtin_datasources import gsi_disaportal

id = "owntwin/sample_modules/bousai"

module = {
    "name": "防災・災害対応",
    "description": "レイヤーは国土地理院ハザードマップポータルサイト「重ねるハザードマップ」のオープンデータです。凡例は <https://disaportal.gsi.go.jp/hazardmap/copyright/opendata.html> を参照してください。",
    "license": "ハザードマップポータルサイト <https://disaportal.gsi.go.jp/hazardmap/copyright/opendata.html>",
    "actions": [
        {
            "type": "link",
            "text": "この場所の防災マニュアルを見る",
            "href": None,
            "params": [],
        },
        {
            "type": "link",
            "text": "この場所から安否状況を登録（発災時のみ）",
            "href": None,
            "params": [],
        },
    ],
    "layers": [
        {
            "id": "kk_keikai",
            "name": "土砂災害警戒区域（急傾斜地の崩壊）",
            "path": "assets/kk_keikai.png",
            "format": "png",
            "enabled": True,
        },
        {
            "id": "kk_houkai",
            "name": "急傾斜地崩壊危険箇所",
            "path": "assets/kk_houkai.png",
            "format": "png",
            "enabled": True,
        },
        {
            "id": "hightide",
            "name": "高潮浸水想定区域",
            "path": "assets/hightide.png",
            "format": "png",
            "enabled": True,
        },
    ],
}


def add(bbox, package, cache_dir):
    basemap_zoom = 18
    tiles = mercantile.tiles(*bbox, basemap_zoom)  # left, bottom, right, top
    tiles = list(tiles)
    # print(tiles)
    # ul = mercantile.ul(tiles[0])
    # br = [mercantile.bounds(tiles[-1]).east, mercantile.bounds(tiles[-1]).south]
    # basemap_bbox = [*ul, *br]
    basemap_bbox = utils.tiles_bounds(tiles)
    logger.info(("basemap_bbox", basemap_bbox))

    zoom = 16
    tiles = mercantile.tiles(*basemap_bbox, zoom)  # left, bottom, right, top
    # print(list(tiles))
    # tiles = mercantile.simplify(tiles)
    tiles = list(tiles)
    # print(tiles)
    layer_bbox = utils.tiles_bounds(tiles)
    logger.info(("layer_bbox", layer_bbox))

    dl = gsi_disaportal.Downloader(cache_dir)

    tiledata = dl.download_kyukeisha_keikai(tiles)
    merged = utils.stitch_tiles(tiledata)
    cropped = utils.crop_img(merged, layer_bbox, basemap_bbox)
    cropped.save(package.assets.joinpath("kk_keikai.png"), "png")

    tiledata = dl.download_kyukeisha_houkai(tiles)
    merged = utils.stitch_tiles(tiledata)
    cropped = utils.crop_img(merged, layer_bbox, basemap_bbox)
    cropped.save(package.assets.joinpath("kk_houkai.png"), "png")

    tiledata = dl.download_hightide(tiles)
    merged = utils.stitch_tiles(tiledata)
    cropped = utils.crop_img(merged, layer_bbox, basemap_bbox)
    cropped.save(package.assets.joinpath("hightide.png"), "png")



import codecs
import json
import os
import shutil
from importlib import import_module
from pathlib import Path
from typing import List

# import importlib.resources  # Strangely not working
import importlib_resources  # Should be replaced with importlib.resource
import mercantile
import owntwin.builder.utils as utils
import typer
from geopy.geocoders import Nominatim
from halo import Halo
from InquirerPy import inquirer
from InquirerPy.separator import Separator
from InquirerPy.validator import NumberValidator
from loguru import logger
from owntwin.builder.package import Package
from owntwin.builder.terrain import extract_meshed_level
from owntwin.builtin_datasources import gsi
from xdg import xdg_cache_home

app = typer.Typer()

FILENAME = "twin.json"

if os.name == "nt":
    CACHE_DIR = Path(os.path.expandvars("%APPDATA%/owntwin/cache"))
else:
    CACHE_DIR = xdg_cache_home().joinpath("owntwin/")

if not CACHE_DIR.exists():
    CACHE_DIR.mkdir(parents=True)


def load_config():
    with open(FILENAME, "r") as f:
        twin = json.load(f)
    return twin


def save_config(config, path):
    # NOTE: Use codecs.open for win
    with codecs.open(path, "w", "utf-8") as f:
        json.dump(config, f, ensure_ascii=False, indent=2)


@app.command("init")
def init(dirname: str = typer.Argument(".")):
    dirname = Path(dirname)

    basemap_zoom = 18  # TODO: Fix

    if Path(dirname.joinpath(FILENAME)).exists():
        typer.Abort()

    if not dirname.is_dir():
        typer.Abort()

    shortdirname = dirname.resolve().name

    uid = inquirer.text(message="Twin ID:", default=shortdirname).execute()

    location_keywords = None
    location = {
        "latitude": None,
        "longitude": None,
        "address": None,
    }
    bbox = []
    default_name = ""

    while True:
        location_keywords = inquirer.text(
            message="Location keywords (leave blank to skip):",
        ).execute()

        if not location_keywords:
            break

        geolocator = Nominatim(user_agent="owntwin-cli")
        location_candidates = geolocator.geocode(
            location_keywords, exactly_one=False, limit=5
        )

        if not location_candidates:
            typer.echo("Not found. Please Try again.")
            continue

        location = inquirer.select(
            message="Select nearby location:",
            choices=list(
                map(lambda x: {"name": x.address, "value": x}, location_candidates)
            )
            + [Separator(), {"name": "(Back)", "value": False}],
        ).execute()

        if location:
            break

    if location_keywords:
        lat, lng = location.latitude, location.longitude
        # print(location, lat, lng)
        size = inquirer.text(
            message="Side length (meter):",
            filter=lambda x: float(x),
            validate=NumberValidator(float_allowed=True),
        ).execute()

        # bbox = [
        #     lng + utils.meter_to_lng(size, lat, lng),  # east
        #     lat - utils.meter_to_lat(size, lat, lng),  # south
        #     lng - utils.meter_to_lng(size, lat, lng),  # west
        #     lat + utils.meter_to_lat(size, lat, lng),  # north
        # ]
        bbox = [
            lng - utils.meter_to_lng(size, lat, lng),  # west
            lat - utils.meter_to_lat(size, lat, lng),  # south
            lng + utils.meter_to_lng(size, lat, lng),  # east
            lat + utils.meter_to_lat(size, lat, lng),  # north
        ]
        tiles = mercantile.tiles(*bbox, basemap_zoom)
        tiles = list(tiles)
        basemap_bbox = utils.tiles_bounds(tiles)
        bbox = basemap_bbox

        typer.echo(
            "  Left (lng): {}\n  Bottom (lat): {}\n  Right (lng): {}\n  Top (lat): {}".format(
                *bbox
            )
        )
        typer.confirm("Is this OK?", default=True, abort=True)
        default_name = location.address

    name = inquirer.text(message="Twin name:", default=default_name).execute()
    type = inquirer.text(message="Twin type:", default="").execute()
    iri = inquirer.text(
        message="IRI or URL:", default=f"https://beta.owntwin.com/twin/{uid}"
    ).execute()
    description = inquirer.text(message="Description:", default="").execute()

    twin = {
        "id": uid,
        "name": name,
        "type": type,
        "type_label": type,
        "iri": iri,
        "description": description,
        "bbox": bbox,
        "canvas": {
            "width": 1024,
            "height": 1024,
        },
        "terrain": {
            "path": "assets/levelmap.json",
        },
        "building": {
            "path": "assets/buildings.json",
        },
        "modules": {},
    }

    # twin = {
    #     "name": name,
    #     ...
    #     "terrain": "levelmap.json",
    #     "objects": {
    #       "building": "building.json",
    #     },
    #     "modules": {},
    # }

    typer.echo(f"About to create {FILENAME}:\n")
    typer.echo(json.dumps(twin, ensure_ascii=False, indent=2))
    typer.confirm("Is this OK?", default=True, abort=True)

    if not dirname.exists():
        dirname.mkdir()

    save_config(twin, dirname.joinpath(FILENAME))


@app.command("add-terrain")
def add_terrain():
    twin = load_config()

    bbox = twin["bbox"]
    package = Package(".")

    dl = gsi.Downloader(CACHE_DIR)  # TODO: Fix

    basemap_zoom = 18
    tiles = mercantile.tiles(*bbox, basemap_zoom)  # left, bottom, right, top
    tiles = list(tiles)

    spinner = Halo(text="", spinner="bouncingBar")
    spinner.start(
        "Installing {}".format(typer.style("terrain", fg=typer.colors.GREEN, bold=True))
    )

    filenames = dl.download_dem5a(tiles)
    merged = utils.geojson_merge(filenames)
    # print(merged.head())

    merged.to_file(package.assets.joinpath("merged_dem5a.geojson"), driver="GeoJSON")

    extract_meshed_level(merged, outfile=package.assets.joinpath("levelmap.json"))

    spinner.succeed()
    spinner.stop()


@app.command("add")
def add(module_names: List[str]):
    # NOTE: a module name can be:
    # local file, PyPI name, git repo/GitHub?

    twin = load_config()

    modules_append = []

    builtin_modules_map = {
        "owntwin.base": "owntwin.builtin_modules.base",
    }

    spinner = Halo(text="", spinner="bouncingBar")
    for module_name in module_names:
        module_name = builtin_modules_map.get(module_name, module_name)
        try:
            module = import_module(module_name)
            spinner.start(
                "Installing {}".format(
                    typer.style(module.id, fg=typer.colors.GREEN, bold=True)
                )
            )
            modules_append.append((module.id, module.module))
            package = Package(".")
            module.add(
                twin["bbox"],
                package,
                CACHE_DIR,
            )
            spinner.succeed()
        except Exception as err:
            logger.error(err)
            spinner.fail()

    spinner.stop()

    if modules_append:
        for (module_name, module) in modules_append:
            twin["modules"][module_name] = module
        save_config(twin, FILENAME)


@app.command("export")
def export(dirname: str = typer.Argument(".")):
    # https://importlib-resources.readthedocs.io/en/latest/migration.html
    path = importlib_resources.files("owntwin.viewer.owntwin")
    spinner = Halo(text="Exporting...", spinner="bouncingBar")
    for entry in path.iterdir():
        logger.info(entry)
        if entry.is_dir():
            shutil.copytree(
                entry,
                Path(dirname).joinpath(entry.name),
                dirs_exist_ok=True,  # TODO: Confirm
                copy_function=shutil.copy,
                ignore=shutil.ignore_patterns(".DS_Store"),
            )
        else:
            if not any(
                [
                    entry.match(exclude)
                    for exclude in ["twin.json", "assets", ".DS_Store"]
                ]
            ):
                shutil.copy(entry, dirname)
    spinner.succeed(text="Done!")
    spinner.stop()


@app.callback()
def main():
    pass


if __name__ == "__main__":
    app()



import io

from loguru import logger
import svgwrite
from scour import scour


def render(gdf, outfile, filter=None, optimize=True):
    logger.info(gdf.crs)
    gdf = gdf.to_crs(epsg=3857)  # 3395

    logger.info(gdf.head())

    minx, miny, maxx, maxy = gdf.geometry.total_bounds
    gdf.geometry = gdf.geometry.translate(-minx, -miny)
    minx, miny, maxx, maxy = gdf.geometry.total_bounds
    logger.info((minx, miny, maxx, maxy))

    # gdf = gdf.scale(100000, 100000, origin=(0, 0))

    viewbox = " ".join(map(str, gdf.total_bounds))
    dwg = svgwrite.Drawing(outfile, height="100%", width="100%", viewBox=(viewbox))

    white = "#FFFFFF"
    grey = "#969696"
    dwg.fill(color=white)
    dwg.stroke(color=grey, width=1)
    extras = {}

    if filter:
        filtered_gdf = gdf[filter]
    else:
        filtered_gdf = gdf

    for g in filtered_gdf.geometry:
        mp = [(x, maxy - y) for x, y in zip(*g.coords.xy)]

        # gr = svgwrite.container.Group(**extras)
        dp = dwg.polyline(points=mp)
        # gr.add(dp)
        dwg.add(dp)

    if optimize:

        class ScourOptions:
            quiet = True

        buf1 = io.StringIO()
        dwg.write(buf1)
        buf1.seek(0)
        buf1 = io.BytesIO(buf1.getvalue().encode())
        buf1.seek(0)
        buf1.name = outfile
        with open(outfile, "wb") as f:
            scour.start(ScourOptions, buf1, f)
    else:
        with open(outfile, "w", encoding="utf-8") as f:
            dwg.write(f)


def render_full(gdf, outfile="full.svg"):
    render(gdf, outfile)


def render_contour(gdf, outfile="contour.svg"):
    render(gdf, outfile, filter=(gdf["class"] == "Cntr"))



import math

from loguru import logger
import geopandas as gpd
import geopy.distance
import mercantile
from PIL import Image


def meter_to_lat(meter, lat, lng):
    lat_dist = geopy.distance.distance((lat, lng), (lat + 1 / 111111, lng)).m
    approx_degree_per_meter = (1 / 111111) / lat_dist
    return meter * approx_degree_per_meter


def meter_to_lng(meter, lat, lng):
    lat_radians = math.radians(lat)
    lng_dist = geopy.distance.distance(
        (lat, lng), (lat, lng + 1 / (111111 * math.cos(lat_radians)))
    ).m
    approx_degree_per_meter = (1 / (111111 * math.cos(lat_radians))) / lng_dist
    return meter * approx_degree_per_meter


def tiles_bounds(tiles):
    tile_ul = min(tiles, key=lambda t: (t.y, t.x))
    tile_br = max(tiles, key=lambda t: (t.y, t.x))
    ul = [mercantile.bounds(tile_ul).west, mercantile.bounds(tile_ul).north]
    br = [mercantile.bounds(tile_br).east, mercantile.bounds(tile_br).south]
    bbox = [ul[0], br[1], br[0], ul[1]]

    # l = float('inf')
    # b = float('inf')
    # r = 0
    # u = 0
    # for tile in tiles:
    #     l = min(l, mercantile.bounds(tile).west)
    #     b = min(b, mercantile.bounds(tile).south)
    #     r = max(r, mercantile.bounds(tile).east)
    #     u = max(u, mercantile.bounds(tile).north)
    # bbox = [l, b, r, u]

    return bbox


def geojson_merge(geojson_filenames):
    if geojson_filenames:
        gdf = gpd.read_file(geojson_filenames[0])
    else:
        return None
    for filename in geojson_filenames[1:]:
        gdf = gdf.append(gpd.read_file(filename))
    return gdf


def make_error_tile(size):
    # cf. https://github.com/geopandas/contextily/pull/69/files
    im = Image.new(mode="RGBA", size=size, color=(0, 0, 0, 0))
    return im


def stitch_tiles(tiledata, size=(256, 256)):
    # print(tiledata)
    xs = list(map(lambda x: x.x, tiledata))
    ys = list(map(lambda x: x.y, tiledata))
    origin_x = min(xs)
    origin_y = min(ys)
    w = size[0] * (max(xs) - min(xs) + 1)
    h = size[1] * (max(ys) - min(ys) + 1)
    # print(w, h)
    dst = Image.new(mode="RGBA", size=(w, h), color=(0, 0, 0, 0))
    for tile in tiledata:
        im = Image.open(tile.filename)
        # print((tile["x"] - origin_x, tile["y"] - origin_y))
        dst.paste(im, (size[0] * (tile.x - origin_x), size[1] * (tile.y - origin_y)))
    return dst


def crop_img(im, img_bbox, crop_bbox, size=(256, 256)):
    # NOTE: Be aware, larger lat, further north
    # TODO: Fix
    # print(im.size)
    logger.info(img_bbox)
    logger.info(crop_bbox)
    img_ul = (img_bbox[0], img_bbox[3])
    img_br = (img_bbox[2], img_bbox[1])
    px_per_lng = im.size[0] / (img_br[0] - img_ul[0])
    px_per_lat = im.size[1] / (-1 * (img_br[1] - img_ul[1]))
    logger.info((px_per_lng, px_per_lat))

    crop_ul = (crop_bbox[0], crop_bbox[3])
    crop_br = (crop_bbox[2], crop_bbox[1])
    relative_crop_ul = (crop_ul[0] - img_ul[0], -1 * (crop_ul[1] - img_ul[1]))
    relative_crop_br = (crop_br[0] - img_ul[0], -1 * (crop_br[1] - img_ul[1]))
    logger.info((img_ul, crop_ul, crop_br))
    logger.info((relative_crop_ul, relative_crop_br))
    crop_bbox_in_px = (
        relative_crop_ul[0] * px_per_lng,
        relative_crop_ul[1] * px_per_lat,
        relative_crop_br[0] * px_per_lng,
        relative_crop_br[1] * px_per_lat,
    )
    logger.info(
        (
            crop_bbox_in_px,
            crop_bbox_in_px[2] - crop_bbox_in_px[0],
            crop_bbox_in_px[3] - crop_bbox_in_px[1],
        )
    )
    cropped = im.crop(crop_bbox_in_px)
    return cropped



import json
from collections import defaultdict

from loguru import logger


def extract_meshed_level(gdf, outfile="levelmap.json"):
    nx_seg = 100
    ny_seg = 100

    logger.info(gdf.crs)
    gdf = gdf.to_crs(epsg=3857)  # 3395

    logger.info(gdf.head())

    minx, miny, maxx, maxy = gdf.geometry.total_bounds
    gdf.geometry = gdf.geometry.translate(-minx, -miny)
    minx, miny, maxx, maxy = gdf.geometry.total_bounds

    bx = maxx / nx_seg
    by = maxy / ny_seg
    logger.info(minx, miny, maxx, maxy, bx, by)

    res = []
    m = defaultdict(lambda: 0)
    mc = defaultdict(lambda: 0)

    for i, row in gdf.iterrows():
        p = row["geometry"]
        alti = row["alti"]
        if alti > 0:
            x = p.x // bx  # - nx_seg // 2
            y = p.y // by  # - ny_seg // 2
            m[(x, y)] += alti
            mc[(x, y)] += 1

    for x in range(nx_seg):
        for y in range(ny_seg):
            if (x, y) in m:
                alti = m[(x, y)]
                res.append([x, y, alti / mc[(x, y)]])
            else:
                res.append([x, y, -1])

    with open(outfile, "w") as f:
        json.dump(res, f)



from pathlib import Path
from time import sleep

import requests
from loguru import logger

FGD_URL = "https://cyberjapandata.gsi.go.jp/xyz/experimental_fgd/{z}/{x}/{y}.geojson"
DEM5A_URL = (
    "https://cyberjapandata.gsi.go.jp/xyz/experimental_dem5a/{z}/{x}/{y}.geojson"
)
DEM10B_URL = (
    "https://cyberjapandata.gsi.go.jp/xyz/experimental_dem10b/{z}/{x}/{y}.geojson"
)


class Downloader(object):
    def __init__(self, cwd, interval=1):
        self.cwd = Path(cwd)
        self.interval = interval

    def download_fgd(self, tiles, cache=True):
        filenames = []

        for i, tile in enumerate(tiles):
            url = FGD_URL.format(z=tile.z, x=tile.x, y=tile.y)

            filename = "fgd-{z}_{x}_{y}.geojson".format(z=tile.z, x=tile.x, y=tile.y)
            filename = self.cwd.joinpath(filename)
            filenames.append(filename)

            logger.info(f"({i + 1}/{len(tiles)}) {url} → {filename}")
            # print(f"({i + 1}/{len(tiles)}) {url} → {str(filename).replace('nolze', 'ozekik')}")
            if cache and filename.exists():
                continue

            resp = requests.get(url)

            with open(filename, "wb") as f:
                f.write(resp.content)

            if len(tiles) - (i + 1) == 0:
                continue

            sleep(self.interval)

        return filenames

    def download_dem5a(self, tiles, cache=True):
        filenames = []

        for i, tile in enumerate(tiles):
            url = DEM5A_URL.format(z=tile.z, x=tile.x, y=tile.y)

            filename = "dem5a-{z}_{x}_{y}.geojson".format(z=tile.z, x=tile.x, y=tile.y)
            filename = self.cwd.joinpath(filename)
            filenames.append(filename)
            logger.info(f"({i + 1}/{len(tiles)}) {url} → {filename}")
            # print(f"({i + 1}/{len(tiles)}) {url} → {str(filename).replace('nolze', 'ozekik')}")
            if cache and filename.exists():
                continue

            resp = requests.get(url)

            with open(filename, "wb") as f:
                f.write(resp.content)

            if len(tiles) - (i + 1) == 0:
                continue

            sleep(self.interval)

        return filenames

    def download_dem10b(self, tiles, cache=True):
        for i, tile in enumerate(tiles):
            url = DEM10B_URL.format(z=tile.z, x=tile.x, y=tile.y)

            filename = "./dem10b-{z}_{x}_{y}.geojson".format(
                z=tile.z, x=tile.x, y=tile.y
            )
            filename = self.cwd.joinpath(filename)
            logger.info((url, filename))
            if cache and filename.exists():
                continue

            resp = requests.get(url)

            with open(filename, "wb") as f:
                f.write(resp.content)

            if len(tiles) - (i + 1) == 0:
                continue

            sleep(self.interval)



from pathlib import Path
from time import sleep

import owntwin.builder.utils as utils
import requests
from loguru import logger
from owntwin.builder.tile import TileData

KK_KEIKAI_URL = (
    "https://disaportaldata.gsi.go.jp/raster/05_kyukeishakeikaikuiki/{z}/{x}/{y}.png"
)
KK_HOUKAI_URL = (
    "https://disaportaldata.gsi.go.jp/raster/05_kyukeisyachihoukai/{z}/{x}/{y}.png"
)
HIGHTIDE_URL = "https://disaportaldata.gsi.go.jp/raster/03_hightide_l2_shinsuishin_data/{z}/{x}/{y}.png"


class Downloader(object):
    def __init__(self, cwd, interval=1):
        self.cwd = Path(cwd)
        self.interval = interval

    def _download_base(self, template_url, template_filename, tiles, cache=True):
        tiledata = []

        z_min, z_max = 2, 17

        for i, tile in enumerate(tiles):
            if not z_min <= tile.z <= z_max:
                raise Exception

            url = template_url.format(z=tile.z, x=tile.x, y=tile.y)

            filename = template_filename.format(z=tile.z, x=tile.x, y=tile.y)
            filename = self.cwd.joinpath(filename)

            td = TileData(filename, z=tile.z, x=tile.x, y=tile.y)
            tiledata.append(td)

            logger.info(f"({i + 1}/{len(tiles)}) {url} → {filename}")
            # logger.info(f"({i + 1}/{len(tiles)}) {url} → {str(filename).replace('nolze', 'ozekik')}")
            if cache and filename.exists():
                continue

            resp = requests.get(url)

            if resp.status_code == 404:
                im = utils.make_error_tile((256, 256))
                im.save(filename, "png")
            else:
                with open(filename, "wb") as f:
                    f.write(resp.content)

            if len(tiles) - (i + 1) == 0:
                continue

            sleep(self.interval)

        return tiledata

    def download_kyukeisha_keikai(self, tiles, cache=True):
        return self._download_base(
            KK_KEIKAI_URL, "kk_keikai-{z}_{x}_{y}.png", tiles, cache=cache
        )

    def download_kyukeisha_houkai(self, tiles, cache=True):
        return self._download_base(
            KK_HOUKAI_URL, "kk_houkai-{z}_{x}_{y}.png", tiles, cache=cache
        )

    def download_hightide(self, tiles, cache=True):
        return self._download_base(
            HIGHTIDE_URL, "hightide-{z}_{x}_{y}.png", tiles, cache=cache
        )



id = "owntwin/sample_modules/jpsearch"

module = {
    "name": "ジャパンサーチ",
    "description": None,
    "license": "https://jpsearch.go.jp/policy",
    "actions": [
        {
            "type": "link",
            "text": "場所名でジャパンサーチを検索",
            "href": "https://jpsearch.go.jp/csearch/jps-cross",
            "default_param": "csid=jps-cross",
            "params": [["q-loc", "name"]],
        },
        {
            "type": "link",
            "text": "位置情報でジャパンサーチを検索",
            "href": "https://beta.owntwin.com/resolver/jpsearch-geohash",
            "params": [["geohash", "geohash"]],
        },
    ],
    "layers": [],
}


def add(bbox, package, cache_dir):
    pass



import mercantile
import owntwin.builder.utils as utils
from loguru import logger
from owntwin.builder.render import render_full
from owntwin.builtin_datasources import gsi

id = "owntwin/base"

module = {
    "name": "基本",
    "description": """国土地理院ベクトルタイル提供実験による「基盤地図情報_基本項目」に基づくベースマップを含む基本モジュールです。

<https://github.com/gsi-cyberjapan/vector-tile-experiment>""",
    "license": "https://github.com/gsi-cyberjapan/vector-tile-experiment#%E6%8F%90%E4%BE%9B%E3%81%AE%E4%BD%8D%E7%BD%AE%E3%81%A5%E3%81%91",
    "actions": [
        {
            "type": "link",
            "text": "この場所の情報修正を提案",
            "href": None,
            "default_params": [],
            "params": [],
        }
    ],
    "layers": [
        {
            "id": "basemap",
            "name": "ベースマップ",
            "path": "assets/basemap.svg",
            "format": "svg",
            "levels": [0],
            "enabled": True,
        },
    ],
}


def add(bbox, package, cache_dir):
    basemap_zoom = 18
    tiles = mercantile.tiles(*bbox, basemap_zoom)  # left, bottom, right, top
    tiles = list(tiles)
    # print(tiles)
    # ul = mercantile.ul(tiles[0])
    # br = [mercantile.bounds(tiles[-1]).east, mercantile.bounds(tiles[-1]).south]
    # basemap_bbox = [*ul, *br]
    basemap_bbox = utils.tiles_bounds(tiles)
    logger.info(("basemap_bbox", basemap_bbox))

    dl = gsi.Downloader(cache_dir)
    filenames = dl.download_fgd(tiles)

    # filenames = []
    # for filename in Path(dl.cwd).glob("fgd-18*.geojson"):
    #     filenames.append(filename)

    merged = utils.geojson_merge(filenames)
    logger.info(merged.head())

    merged.to_file(package.assets.joinpath("merged.geojson"), driver="GeoJSON")

    render_full(merged, outfile=package.assets.joinpath("basemap.svg"))



from pathlib import Path


class Package(object):
    def __init__(self, cwd):
        self.cwd = Path(cwd)
        if not self.cwd.exists():
            self.cwd.mkdir()
        assert self.cwd.is_dir()

        self.assets = self.cwd.joinpath("./assets/")
        if not self.assets.exists():
            self.assets.mkdir()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['owntwin',
 'owntwin.builder',
 'owntwin.builtin_datasources',
 'owntwin.builtin_modules.base',
 'owntwin.sample_modules.bousai',
 'owntwin.sample_modules.jpsearch',
 'owntwin.viewer']

package_data = \
{'': ['*'],
 'owntwin.viewer': ['owntwin/asset-manifest.json',
                    'owntwin/asset-manifest.json',
                    'owntwin/index.html',
                    'owntwin/index.html',
                    'owntwin/static/css/*',
                    'owntwin/static/js/*']}

install_requires = \
['InquirerPy>=0.1.0,<0.2.0',
 'Pillow>=8.0.1,<9.0.0',
 'colorama>=0.4.4,<0.5.0',
 'geopandas>=0.8.1,<0.9.0',
 'geopy>=2.0.0,<3.0.0',
 'halo>=0.0.31,<0.0.32',
 'importlib-resources>=5.1.0,<6.0.0',
 'loguru>=0.5.3,<0.6.0',
 'mercantile>=1.1.6,<2.0.0',
 'requests>=2.24.0,<3.0.0',
 'scour>=0.38.1,<0.39.0',
 'svgwrite>=1.4,<2.0',
 'typer>=0.3.2,<0.4.0',
 'xdg>=5.0.1,<6.0.0']

entry_points = \
{'console_scripts': ['owntwin = owntwin.cli:app']}

setup_kwargs = {
    'name': 'owntwin',
    'version': '0.1.2',
    'description': 'User-driven digital twin framework.',
    'long_description': '# OwnTwin CLI\n\nOwnTwin CLI is a toolchain for OwnTwin, a user-driven digital twin framework.\n\n## Installation\n\n```\npip install owntwin\n```\n\n## Usage\n\nSee the [documentation](https://beta.owntwin.com/docs/getting-started) for details.\n\n```\nowntwin init\nowntwin add-terrain\nowntwin add [module names...]\nowntwin export\n```\n\n```\nowntwin view\n```\n',
    'author': 'Kentaro Ozeki',
    'author_email': 'kentaro.ozeki+dev@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/owntwin/owntwin-cli',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import os
import posixpath
import re
import sys
import urllib
from functools import partial
from pathlib import Path

import importlib_resources
import typer
from loguru import logger

import owntwin.builder.cli

logger.remove()
logger.add(sys.stderr, format="{message}", level="ERROR", backtrace=True, diagnose=True)
# logger.add(sys.stderr, format="{message}", level="INFO")

app = typer.Typer()

# app.add_typer(owntwin.builder.cli.app, name="builder")

app.command("init")(owntwin.builder.cli.init)
app.command("add-terrain")(owntwin.builder.cli.add_terrain)
app.command("add")(owntwin.builder.cli.add)
app.command("export")(owntwin.builder.cli.export)


@app.command("view")
def view(
    dirname: str = typer.Argument("."),
    port: int = typer.Option(8000),
    no_terrain: bool = typer.Option(False, "--no-terrain", help="Disable terrain"),
):
    import socketserver
    from http.server import SimpleHTTPRequestHandler

    class RequestHandler(SimpleHTTPRequestHandler):
        def log_message(self, format, *args):
            pass

        def end_headers(self):
            self.send_header("Access-Control-Allow-Origin", "*")
            self.send_header("Cache-Control", "no-cache, no-store, must-revalidate")
            self.send_header("Pragma", "no-cache")
            self.send_header("Expires", "0")
            SimpleHTTPRequestHandler.end_headers(self)

        def translate_path(self, path):
            if path.startswith("/__/"):
                path = re.sub(r"^/__/", "/", path)
                directory = str(Path(dirname))
            else:
                directory = self.directory
            # print(path, directory)
            # abandon query parameters
            path = path.split("?", 1)[0]
            path = path.split("#", 1)[0]
            # Don't forget explicit trailing slash when normalizing. Issue17324
            trailing_slash = path.rstrip().endswith("/")
            try:
                path = urllib.parse.unquote(path, errors="surrogatepass")
            except UnicodeDecodeError:
                path = urllib.parse.unquote(path)
            path = posixpath.normpath(path)
            words = path.split("/")
            words = filter(None, words)
            path = directory
            for word in words:
                if os.path.dirname(word) or word in (os.curdir, os.pardir):
                    # Ignore components that are not a simple file/directory name
                    continue
                path = os.path.join(path, word)
            if trailing_slash:
                path += "/"
            return path

    Handler = partial(
        RequestHandler,
        # directory=Path(__file__).parent.joinpath("./viewer/owntwin/"),
        directory=importlib_resources.files("owntwin.viewer.owntwin").joinpath("."),
    )

    Server = socketserver.TCPServer
    Server.allow_reuse_address = True

    with Server(("localhost", port), Handler) as httpd:
        print(
            f"Started preview on http://localhost:{port}/?twin=http://localhost:{port}/__/{'&no-terrain' if no_terrain else ''}"
        )
        httpd.serve_forever()


@app.callback()
def main():
    pass


if __name__ == "__main__":
    app()
